/*
 * Copyright 2013 Red Hat Inc., Durham, North Carolina.
 * All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Authors:
 * 	Šimon Lukašík
 */

#ifndef _OSCAP_CPEDICT_EXT_PRIV_H_
#define _OSCAP_CPEDICT_EXT_PRIV_H_

#include <libxml/xmlreader.h>
#include <libxml/xmlwriter.h>

#include "cpe_ctx_priv.h"
#include "common/util.h"


#define TAG_CPE23_ITEM_STR		"cpe23-item"

struct cpe23_item;			///< Represents <cpe_dict_ext:cpe23-item> node

/**
 * Parse cpe_dict_ext:cpe23-item element from xmlTextReader
 * @param reader xmlTextReader
 * @returns newly parser cpe23-item representation
 */
struct cpe23_item *cpe23_item_parse(xmlTextReaderPtr reader);

/**
 * Export cpe_dict_ext:cpe23-item representation to xmlTextWriter
 * @param item cpe23-item to export
 * @param writer xmlTextWriter
 * @returns zero on success
 */
int cpe23_item_export(const struct cpe23_item *item, xmlTextWriterPtr writer);

/**
 * Dispose cpe23-item representation
 * @param item cpe23-item to dispose
 */
void cpe23_item_free(struct cpe23_item *item);


#endif
