/*
 * Copyright 2013 Red Hat Inc., Durham, North Carolina.
 * All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Authors:
 *   Simon Lukasik <slukasik@redhat.com>
 *
 */

#pragma once
#ifndef _OSCAP_XMLNS_H
#define _OSCAP_XMLNS_H


/// CPE version 2 Language XML namespace
#define XMLNS_CPE2L		"http://cpe.mitre.org/language/2.0"
/// CPE version 1 dictionary XML namespace
#define XMLNS_CPE1D		"http://cpe.mitre.org/XMLSchema/cpe/1.0"
/// CPE version 2 dictionary XML namespace
#define XMLNS_CPE2D		"http://cpe.mitre.org/dictionary/2.0"
/// CPE version 2 dictionary metadata XML namespace
#define XMLNS_CPE2D_METADATA	"http://scap.nist.gov/schema/cpe-dictionary-metadata/0.2"
/// CPE version 2.3 extension
#define XMLNS_CPE2D3_EXTENSION	"http://scap.nist.gov/schema/cpe-extension/2.3"


#endif
