tr.rule-overview-needs-attention td a { color: #d9534f }

td.rule-result div, span.rule-result { text-align: center; font-weight: bold; color: #fff; background: #808080 }
td.rule-result-fail div, span.rule-result-fail { background: #d9534f }
td.rule-result-error div, span.rule-result-error { background: #d9534f }
td.rule-result-unknown div, span.rule-result-unknown { background: #f0ad4e }
td.rule-result-pass div, span.rule-result-pass { background: #5cb85c }
td.rule-result-fixed div, span.rule-result-fixed { background: #5cb85c }

.js-only { display: none }

.rule-result-filtered, .rule-result-filtered > * { display: none !important }
.search-no-match, .search-no-match > * { display: none !important }

.rule-detail-fail, .rule-detail-error, .rule-detail-unknown { border: 2px solid #d9534f }

#footer { text-align: center; margin-top: 50px }

pre
{
    overflow: auto !important;
    word-wrap: normal !important;
    white-space: pre-wrap;
}

div.check-system-details, div.remediation, div.description { display: inline-block; width: 0; min-width: 100%; overflow-x: auto }

div.profile-description
{
    white-space: pre-wrap;
}

div.modal-body
{
    margin: 50px;
    padding: 0;
}

div.horizontal-scroll
{
    overflow-x: auto;
}

div.top-spacer-10
{
    margin-top: 10px;
}

@media print
{
    .container { width: 100% }
    /* ":after" instead of "::after" is used for compatibility with IE8 */
    .rule-result abbr[title]:after { content: "" }
    .identifiers abbr[title]:after { content: "" }
    .identifiers a[href]:after { content: "" }
}
